/*
 * Project name:
     Rotary click
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     - initial release (FJ);
 * Description:
     - This project is a simple demonstration of working with the Rotary click board.
     - Encoder pin A is connected to port RA2, pin B is connected to port RC0 and SW to port RB0.
     - Use encoder to rotate LED's on Rotary click board, and SW to turn on one, two, four or eight LEDS.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7 Connectivity
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8MHz Crystal, 32MHz MCU Clock
     ext. modules:    Rotary click board click
                      http://www.mikroe.com/click/rotary/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Place Rotary click board in the mikroBUS socket 1.
     - Pull down RB0 on the EasyPIC Pro v7 board.
 */

sbit HC595_cs           at RE0_bit;
sbit ENCA               at RC0_bit;
sbit ENCB               at RA2_bit;
sbit ENC_SW             at RB0_bit;
sbit HC595_cs_direction at TRISE0_bit;
sbit ENCA_direction     at TRISC0_bit;
sbit ENCB_direction     at TRISA2_bit;
sbit ENC_SW_direction   at TRISB0_bit;

unsigned int Led_State = 0x0001;
char ENCA_bit,ENCB_bit;
char ENC_Button_State = 0;

// ISR
void Interrupt(){
  ENC_Button_State++;                                   // set new ENC button state
  if (ENC_Button_State == 4)
    ENC_Button_State = 0;
  // check for interrupt
  if (INT0IF_bit){
    INT0IF_bit = 0;                                     // clear Interrupt flag
    // check ENC button state
    switch (ENC_Button_State){
      case 0 : Led_State = 0x0001;                      // One LED are ON
               break;
      case 1 : Led_State = 0x0101;                      // Two LEDs are ON
               break;
      case 2 : Led_State = 0x1111;                      // Four LEDs are ON
               break;
      case 3 : Led_State = 0x5555;                      // Eight LEDs are ON
               break;
      default :
               break;
    }
  }
}

void HC595_Write(int value){
  char first_byte, second_byte;
  
  second_byte = value;
  value = value>>8;
  first_byte = value;

  SPI1_write(first_byte);
  SPI1_write(second_byte);
  
  HC595_cs = 0;
  asm{nop};
  asm{nop};
  asm{nop};
  HC595_cs = 1;
}

void main() {
  ANSELA = 0 ;                                          // Configure PORTA pins as digital
  ANSELB = 0 ;                                          // Configure PORTB pins as digital
  ANSELC = 0 ;                                          // Configure PORTC pins as digital
  
  ENCA_direction     = 1;                               // Configure RC0 as input
  ENCB_direction     = 1;                               // Configure RA2 as input
  ENC_SW_direction   = 1;                               // Configure RB0 as input
  HC595_cs_direction = 0;                               // Configure RE0 as output
  
  INT0IE_bit = 1 ;                                      // Enable INTE interrupt on RB0
  GIE_bit = 1 ;                                         // Enable Global Interrupt
  
  SPI1_Init();                                          // Initialize SPI1
  
  while(1) {
    if((ENCA_bit != RC0_bit)||(ENCB_bit != RA2_bit)){   // If logic state on pin A or pin B change
                                                        // Compare last logic state of pin B and new logick state on pin A
      if(ENCB_bit|RC0_bit==1){                          // If ENCB_old OR ENCA_new is 1
        Led_State = (Led_State << 1) | (Led_State >> 15);
      }
                    
      if(ENCA_bit|RA2_bit==1){                          // If ENCA_old OR ENCB_new is 1
        Led_State = (Led_State >> 1) | (Led_State <<15);
      }

      ENCA_bit = RC0_bit ;                              // Set new value of ENCA_bit
      ENCB_bit = RA2_bit ;                              // Set new value of ENCB_bit
    }
    
  HC595_Write(Led_State);
  }
}